@@ -1,26 +1,27 @@
-
-/*
-
-Copyright (C) 1996-2017 John W. Eaton
-Copyright (C) 2009 VZLU Prague
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+/////////////////////////////////////////////////////////////////////////*
+//
+// Copyright (C) 1996-2022 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if defined (HAVE_CONFIG_H)
 #  include "config.h"
@@ -43,10 +44,40 @@ along with Octave; see the file COPYING.  If not, see
 #include "utils.h"
 #include "ov-re-mat.h"
 
+OCTAVE_NAMESPACE_BEGIN
+
 /*
-%!shared __random_statistical_tests__
+%% Restore all rand* "seed" and "state" values in order, so that the
+%% new "state" algorithm remains active after these tests complete.
+%!function restore_rand_states (seed, state)
+%!  rand ("seed", seed.rand);
+%!  rande ("seed", seed.rande);
+%!  randg ("seed", seed.randg);
+%!  randn ("seed", seed.randn);
+%!  randp ("seed", seed.randp);
+%!  rand ("state", state.rand);
+%!  rande ("state", state.rande);
+%!  randg ("state", state.randg);
+%!  randn ("state", state.randn);
+%!  randp ("state", state.randp);
+%!endfunction
+
+%!shared __random_statistical_tests__, old_seed, old_state, restore_state
 %! ## Flag whether the statistical tests should be run in "make check" or not
 %! __random_statistical_tests__ = 0;
+%! ## Save and restore the states of each of the random number generators
+%! ## that are tested by the unit tests in this file.
+%! old_seed.rand = rand ("seed");
+%! old_seed.rande = rande ("seed");
+%! old_seed.randg = randg ("seed");
+%! old_seed.randn = randn ("seed");
+%! old_seed.randp = randp ("seed");
+%! old_state.rand = rand ("state");
+%! old_state.rande = rande ("state");
+%! old_state.randg = randg ("state");
+%! old_state.randn = randn ("state");
+%! old_state.randp = randp ("state");
+%! restore_state = onCleanup (@() restore_rand_states (old_seed, old_state));
 */
 
 static octave_value
@@ -88,12 +119,14 @@ do_rand (const octave_value_list& args, int nargin, const char *fcn,
   octave_value retval;
   dim_vector dims;
 
-  octave::unwind_protect frame;
   // Restore current distribution on any exit.
-  frame.add_fcn (octave_rand::distribution,
-                 octave_rand::distribution ());
+  unwind_action restore_distribution
+    ([] (const std::string& old_distribution)
+     {
+       rand::distribution (old_distribution);
+     }, rand::distribution ());
 
-  octave_rand::distribution (distribution);
+  rand::distribution (distribution);
 
   switch (nargin)
     {
@@ -122,41 +155,37 @@ do_rand (const octave_value_list& args, int nargin, const char *fcn,
             std::string s_arg = tmp.string_value ();
 
             if (s_arg == "dist")
-              retval = octave_rand::distribution ();
+              retval = rand::distribution ();
             else if (s_arg == "seed")
-              retval = octave_rand::seed ();
+              retval = rand::seed ();
             else if (s_arg == "state" || s_arg == "twister")
-              retval = octave_rand::state (fcn);
+              retval = rand::state (fcn);
             else if (s_arg == "uniform")
-              octave_rand::uniform_distribution ();
+              rand::uniform_distribution ();
             else if (s_arg == "normal")
-              octave_rand::normal_distribution ();
+              rand::normal_distribution ();
             else if (s_arg == "exponential")
-              octave_rand::exponential_distribution ();
+              rand::exponential_distribution ();
             else if (s_arg == "poisson")
-              octave_rand::poisson_distribution ();
+              rand::poisson_distribution ();
             else if (s_arg == "gamma")
-              octave_rand::gamma_distribution ();
+              rand::gamma_distribution ();
             else
               error ("%s: unrecognized string argument", fcn);
           }
         else if (tmp.is_scalar_type ())
           {
-            double dval = tmp.double_value ();
-
-            if (octave::math::isnan (dval))
-              error ("%s: NaN is invalid matrix dimension", fcn);
+            octave_idx_type n = tmp.idx_type_value (true);
 
             dims.resize (2);
 
-            dims(0) = octave::math::nint_big (tmp.double_value ());
-            dims(1) = octave::math::nint_big (tmp.double_value ());
+            dims(0) = dims(1) = n;
 
             goto gen_matrix;
           }
         else if (tmp.is_range ())
           {
-            Range r = tmp.range_value ();
+            range<double> r = tmp.range_value ();
 
             if (! r.all_elements_are_ints ())
               error ("%s: all elements of range must be integers", fcn);
@@ -165,13 +194,13 @@ do_rand (const octave_value_list& args, int nargin, const char *fcn,
 
             dims.resize (n);
 
-            octave_idx_type base = octave::math::nint_big (r.base ());
-            octave_idx_type incr = octave::math::nint_big (r.inc ());
+            octave_idx_type base = math::nint_big (r.base ());
+            octave_idx_type incr = math::nint_big (r.increment ());
 
             for (octave_idx_type i = 0; i < n; i++)
               {
                 // Negative dimensions treated as zero for Matlab compatibility
-                dims(i) = base >= 0 ? base : 0;
+                dims(i) = (base >= 0 ? base : 0);
                 base += incr;
               }
 
@@ -179,15 +208,15 @@ do_rand (const octave_value_list& args, int nargin, const char *fcn,
           }
         else if (tmp.is_matrix_type ())
           {
-            Array<int> iv;
+            Array<octave_idx_type> iv;
 
             try
               {
-                iv = tmp.int_vector_value (true);
+                iv = tmp.octave_idx_type_vector_value (true);
               }
-            catch (octave::execution_exception& e)
+            catch (execution_exception& ee)
               {
-                error (e, "%s: dimensions must be a scalar or array of integers", fcn);
+                error (ee, "%s: dimensions must be a scalar or array of integers", fcn);
               }
 
             octave_idx_type len = iv.numel ();
@@ -198,7 +227,7 @@ do_rand (const octave_value_list& args, int nargin, const char *fcn,
               {
                 // Negative dimensions treated as zero for Matlab compatibility
                 octave_idx_type elt = iv(i);
-                dims(i) = elt >=0 ? elt : 0;
+                dims(i) = (elt >=0 ? elt : 0);
               }
 
             goto gen_matrix;
@@ -222,11 +251,11 @@ do_rand (const octave_value_list& args, int nargin, const char *fcn,
                   {
                     double d = args(idx+1).double_value ();
 
-                    octave_rand::seed (d);
+                    rand::seed (d);
                   }
                 else if (args(idx+1).is_string ()
                          && args(idx+1).string_value () == "reset")
-                  octave_rand::reset ();
+                  rand::reset ();
                 else
                   error ("%s: seed must be a real scalar", fcn);
               }
@@ -234,13 +263,19 @@ do_rand (const octave_value_list& args, int nargin, const char *fcn,
               {
                 if (args(idx+1).is_string ()
                     && args(idx+1).string_value () == "reset")
-                  octave_rand::reset (fcn);
+                  rand::reset (fcn);
                 else
                   {
-                    ColumnVector s =
-                      ColumnVector (args(idx+1).vector_value(false, true));
+                    ColumnVector s
+                      = ColumnVector (args(idx+1).vector_value (false, true));
 
-                    octave_rand::state (s, fcn);
+                    // Backwards compatibility with previous versions of
+                    // Octave which mapped Inf to 0.
+                    for (octave_idx_type i = 0; i < s.numel (); i++)
+                      if (math::isinf (s.xelem (i)))
+                        s.xelem (i) = 0.0;
+
+                    rand::state (s, fcn);
                   }
               }
             else
@@ -252,13 +287,10 @@ do_rand (const octave_value_list& args, int nargin, const char *fcn,
 
             for (int i = 0; i < nargin; i++)
               {
-                octave_idx_type elt =
-                  args(idx+i).xidx_type_value (
-                    "%s: dimension must be a scalar or array of integers",
-                    fcn);
+                octave_idx_type elt = args(idx+i).idx_type_value (true);
 
                 // Negative dimensions treated as zero for Matlab compatibility
-                dims(i) = elt >= 0 ? elt : 0;
+                dims(i) = (elt >= 0 ? elt : 0);
               }
 
             goto gen_matrix;
@@ -279,7 +311,7 @@ gen_matrix:
       if (additional_arg)
         {
           if (a.numel () == 1)
-            return octave_rand::float_nd_array (dims, a(0));
+            return rand::float_nd_array (dims, a(0));
           else
             {
               if (a.dims () != dims)
@@ -290,20 +322,20 @@ gen_matrix:
               float *v = m.fortran_vec ();
 
               for (octave_idx_type i = 0; i < len; i++)
-                v[i] = octave_rand::float_scalar (a(i));
+                v[i] = rand::float_scalar (a(i));
 
               return m;
             }
         }
       else
-        return octave_rand::float_nd_array (dims);
+        return rand::float_nd_array (dims);
     }
   else
     {
       if (additional_arg)
         {
           if (a.numel () == 1)
-            return octave_rand::nd_array (dims, a(0));
+            return rand::nd_array (dims, a(0));
           else
             {
               if (a.dims () != dims)
@@ -314,13 +346,13 @@ gen_matrix:
               double *v = m.fortran_vec ();
 
               for (octave_idx_type i = 0; i < len; i++)
-                v[i] = octave_rand::scalar (a(i));
+                v[i] = rand::scalar (a(i));
 
               return m;
             }
         }
       else
-        return octave_rand::nd_array (dims);
+        return rand::nd_array (dims);
     }
 }
 
@@ -360,20 +392,22 @@ You may also initialize the state vector from an arbitrary vector of length
 @leq{} 625 for @var{v}.  This new state will be a hash based on the value of
 @var{v}, not @var{v} itself.
 
-By default, the generator is initialized from @code{/dev/urandom} if it is
-available, otherwise from CPU time, wall clock time, and the current
-fraction of a second.  Note that this differs from @sc{matlab}, which
-always initializes the state to the same state at startup.  To obtain
-behavior comparable to @sc{matlab}, initialize with a deterministic state
-vector in Octave's startup files (@pxref{Startup Files}).
+By default, the generator is initialized by contributing entropy from the
+wall clock time, the CPU time, the current fraction of a second, the process
+ID and---if available---up to 1024 bits from the C++ random numbers source
+@code{random_device}, which might be non-deterministic (implementation
+specific).  Note that this differs from @sc{matlab}, which always initializes
+the state to the same state at startup.  To obtain behavior comparable to
+@sc{matlab}, initialize with a deterministic state vector in Octave's startup
+files (@pxref{Startup Files}).
 
 To compute the pseudo-random sequence, @code{rand} uses the Mersenne
 Twister with a period of @math{2^{19937}-1}
 (See @nospell{M. Matsumoto and T. Nishimura},
 @cite{Mersenne Twister: A 623-dimensionally equidistributed uniform
 pseudorandom number generator},
-ACM Trans. on Modeling and Computer Simulation Vol. 8, No. 1, pp. 3--30,
-January 1998,
+@nospell{ACM} Trans.@: on Modeling and Computer Simulation Vol.@: 8, No.@: 1,
+pp.@: 3--30, January 1998,
 @url{http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html}).
 Do @strong{not} use for cryptography without securely hashing several
 returned values together, otherwise the generator state can be learned after
@@ -428,12 +462,15 @@ classes.
 %! rand ("state", [12,13]);  x = rand (1,4);
 %! rand ("state", [12;13]);  y = rand (1,4);
 %! assert (x, y);
+%!test  # querying "state" returns a value which can be used later
+%! s = rand ("state");  x = rand (1,2);
+%! rand ("state", s);   y = rand (1,2);
+%! assert (x, y);
 %!test  # querying "state" doesn't disturb sequence
 %! rand ("state", 12);  rand (1,2);  x = rand (1,2);
-%! rand ("state", 12);  rand (1,2);
-%! s = rand ("state");  y = rand (1,2);
+%! rand ("state", 12);  rand (1,2);  s = rand ("state");  y = rand (1,2);
 %! assert (x, y);
-%! rand ("state", s);  z = rand (1,2);
+%! rand ("state", s);   z = rand (1,2);
 %! assert (x, z);
 %!test  # "seed" must be a scalar
 %! rand ("seed", 12);  x = rand (1,4);
@@ -442,12 +479,11 @@ classes.
 %!error <seed must be a real scalar> rand ("seed", [12,13])
 %!test  # querying "seed" returns a value which can be used later
 %! s = rand ("seed");  x = rand (1,2);
-%! rand ("seed", s);  y = rand (1,2);
+%! rand ("seed", s);   y = rand (1,2);
 %! assert (x, y);
 %!test  # querying "seed" doesn't disturb sequence
 %! rand ("seed", 12);  rand (1,2);  x = rand (1,2);
-%! rand ("seed", 12);  rand (1,2);
-%! s = rand ("seed");  y = rand (1,2);
+%! rand ("seed", 12);  rand (1,2);  s = rand ("seed");  y = rand (1,2);
 %! assert (x, y);
 %! rand ("seed", s);  z = rand (1,2);
 %! assert (x, z);
@@ -455,20 +491,20 @@ classes.
 
 /*
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! rand ("state", 1);
-%! assert (rand (1,6), [0.1343642441124013 0.8474337369372327 0.763774618976614 0.2550690257394218 0.495435087091941 0.4494910647887382], 1e-6);
+%! assert (rand (1,6), [0.1343642441124013 0.8474337369372327 0.763774618976614 0.2550690257394218 0.495435087091941 0.4494910647887382], eps);
 %!test
-%! ## Test fixed seed
+%! ## Test a known fixed seed
 %! rand ("seed", 1);
 %! assert (rand (1,6), [0.8668024251237512 0.9126510815694928 0.09366085007786751 0.1664607301354408 0.7408077004365623 0.7615650338120759], 1e-6);
 %!test
 %! if (__random_statistical_tests__)
 %!   ## statistical tests may fail occasionally.
 %!   rand ("state", 12);
-%!   x = rand (100000, 1);
-%!   assert (max (x) < 1);   #*** Please report this!!! ***
+%!   x = rand (100_000, 1);
 %!   assert (min (x) > 0);   #*** Please report this!!! ***
+%!   assert (max (x) < 1);   #*** Please report this!!! ***
 %!   assert (mean (x), 0.5, 0.0024);
 %!   assert (var (x), 1/48, 0.0632);
 %!   assert (skewness (x), 0, 0.012);
@@ -478,7 +514,7 @@ classes.
 %! if (__random_statistical_tests__)
 %!   ## statistical tests may fail occasionally.
 %!   rand ("seed", 12);
-%!   x = rand (100000, 1);
+%!   x = rand (100_000, 1);
 %!   assert (max (x) < 1);   #*** Please report this!!! ***
 %!   assert (min (x) > 0);   #*** Please report this!!! ***
 %!   assert (mean (x), 0.5, 0.0024);
@@ -489,19 +525,29 @@ classes.
 */
 
 /*
-%!# Test out-of-range values as rand() seeds.  See oct-rand.cc: double2uint32().
+## Test out-of-range values as rand() seeds.
 %!function v = __rand_sample__ (initval)
 %!  rand ("state", initval);
 %!  v = rand (1, 6);
 %!endfunction
 %!
-%!assert (__rand_sample__ (0), __rand_sample__ (2^32))
-%!assert (__rand_sample__ (-2), __rand_sample__ (2^32-2))
-%!assert (__rand_sample__ (Inf), __rand_sample__ (NaN))
-%!assert (! isequal (__rand_sample__ (-1), __rand_sample__ (-2)))
+%!assert (__rand_sample__ (-1), __rand_sample__ (0))
+%!assert (__rand_sample__ (-Inf), __rand_sample__ (0))
+%!assert (__rand_sample__ (2^33), __rand_sample__ (intmax ("uint32")))
+%!assert (__rand_sample__ (Inf), __rand_sample__ (0))
+%!assert (__rand_sample__ (NaN), __rand_sample__ (0))
 */
 
-static std::string current_distribution = octave_rand::distribution ();
+/*
+## Check that negative dimensions are treated as zero for Matlab compatibility
+%!assert (size (rand (1, -1, 2)), [1, 0, 2])
+
+## Test input validation
+%!error <conversion of 1.1 to.* failed> rand (1, 1.1)
+%!error <dimensions must be .* array of integers> rand ([1, 1.1])
+*/
+
+static std::string current_distribution = rand::distribution ();
 
 DEFUN (randn, args, ,
        doc: /* -*- texinfo -*-
@@ -531,7 +577,7 @@ classes.
 Reference: @nospell{G. Marsaglia and W.W. Tsang},
 @cite{Ziggurat Method for Generating Random Variables},
 J. Statistical Software, vol 5, 2000,
-@url{http://www.jstatsoft.org/v05/i08/}
+@url{https://www.jstatsoft.org/v05/i08/}
 
 @seealso{rand, rande, randg, randp}
 @end deftypefn */)
@@ -541,18 +587,18 @@ J. Statistical Software, vol 5, 2000,
 
 /*
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! randn ("state", 1);
-%! assert (randn (1, 6), [-2.666521678978671 -0.7381719971724564 1.507903992673601 0.6019427189162239 -0.450661261143348 -0.7054431351574116], 1e-6);
+%! assert (randn (1, 6), [-2.666521678978671 -0.7381719971724564 1.507903992673601 0.6019427189162239 -0.450661261143348 -0.7054431351574116], 14*eps);
 %!test
-%! ## Test fixed seed
+%! ## Test a known fixed seed
 %! randn ("seed", 1);
 %! assert (randn (1, 6), [-1.039402365684509 -1.25938892364502 0.1968704611063004 0.3874166905879974 -0.5976632833480835 -0.6615074276924133], 1e-6);
 %!test
 %! if (__random_statistical_tests__)
 %!   ## statistical tests may fail occasionally.
 %!   randn ("state", 12);
-%!   x = randn (100000, 1);
+%!   x = randn (100_000, 1);
 %!   assert (mean (x), 0, 0.01);
 %!   assert (var (x), 1, 0.02);
 %!   assert (skewness (x), 0, 0.02);
@@ -562,7 +608,7 @@ J. Statistical Software, vol 5, 2000,
 %! if (__random_statistical_tests__)
 %!   ## statistical tests may fail occasionally.
 %!   randn ("seed", 12);
-%!   x = randn (100000, 1);
+%!   x = randn (100_000, 1);
 %!   assert (mean (x), 0, 0.01);
 %!   assert (var (x), 1, 0.02);
 %!   assert (skewness (x), 0, 0.02);
@@ -598,7 +644,7 @@ classes.
 Reference: @nospell{G. Marsaglia and W.W. Tsang},
 @cite{Ziggurat Method for Generating Random Variables},
 J. Statistical Software, vol 5, 2000,
-@url{http://www.jstatsoft.org/v05/i08/}
+@url{https://www.jstatsoft.org/v05/i08/}
 
 @seealso{rand, randn, randg, randp}
 @end deftypefn */)
@@ -608,18 +654,18 @@ J. Statistical Software, vol 5, 2000,
 
 /*
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! rande ("state", 1);
-%! assert (rande (1, 6), [3.602973885835625 0.1386190677555021 0.6743112889616958 0.4512830847258422 0.7255744741233175 0.3415969205292291], 1e-6);
+%! assert (rande (1, 6), [3.602973885835625 0.1386190677555021 0.6743112889616958 0.4512830847258422 0.7255744741233175 0.3415969205292291], 2*eps);
 %!test
-%! ## Test fixed seed
+%! ## Test a known fixed seed
 %! rande ("seed", 1);
 %! assert (rande (1, 6), [0.06492075175653866 1.717980206012726 0.4816154008731246 0.5231300676241517 0.103910739364359 1.668931916356087], 1e-6);
 %!test
 %! if (__random_statistical_tests__)
 %!   ## statistical tests may fail occasionally
 %!   rande ("state", 1);
-%!   x = rande (100000, 1);
+%!   x = rande (100_000, 1);
 %!   assert (min (x) > 0);   # *** Please report this!!! ***
 %!   assert (mean (x), 1, 0.01);
 %!   assert (var (x), 1, 0.03);
@@ -630,7 +676,7 @@ J. Statistical Software, vol 5, 2000,
 %! if (__random_statistical_tests__)
 %!   ## statistical tests may fail occasionally
 %!   rande ("seed", 1);
-%!   x = rande (100000, 1);
+%!   x = rande (100_000, 1);
 %!   assert (min (x)>0);   # *** Please report this!!! ***
 %!   assert (mean (x), 1, 0.01);
 %!   assert (var (x), 1, 0.03);
@@ -641,9 +687,9 @@ J. Statistical Software, vol 5, 2000,
 
 DEFUN (randg, args, ,
        doc: /* -*- texinfo -*-
-@deftypefn  {} {} randg (@var{n})
-@deftypefnx {} {} randg (@var{m}, @var{n}, @dots{})
-@deftypefnx {} {} randg ([@var{m} @var{n} @dots{}])
+@deftypefn  {} {} randg (@var{a}, @var{n})
+@deftypefnx {} {} randg (@var{a}, @var{m}, @var{n}, @dots{})
+@deftypefnx {} {} randg (@var{a}, [@var{m} @var{n} @dots{}])
 @deftypefnx {} {@var{v} =} randg ("state")
 @deftypefnx {} {} randg ("state", @var{v})
 @deftypefnx {} {} randg ("state", "reset")
@@ -652,6 +698,7 @@ DEFUN (randg, args, ,
 @deftypefnx {} {} randg ("seed", "reset")
 @deftypefnx {} {} randg (@dots{}, "single")
 @deftypefnx {} {} randg (@dots{}, "double")
+
 Return a matrix with @code{gamma (@var{a},1)} distributed random elements.
 
 The arguments are handled the same as the arguments for @code{rand}, except
@@ -753,43 +800,43 @@ classes.
 %! assert (randg ([-inf, -1, 0, inf, nan]), [nan, nan, nan, nan, nan]); # *** Please report
 
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! randg ("state", 1);
-%! assert (randg (0.1, 1, 6), [0.0103951513331241 8.335671459898252e-05 0.00138691397249762 0.000587308416993855 0.495590518784736 2.3921917414795e-12], 1e-6);
+%! assert (randg (0.1, 1, 6), [0.0103951513331241 8.335671459898252e-05 0.00138691397249762 0.000587308416993855 0.495590518784736 2.3921917414795e-12], eps);
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! randg ("state", 1);
-%! assert (randg (0.95, 1, 6), [3.099382433255327 0.3974529788871218 0.644367450750855 1.143261091802246 1.964111762696822 0.04011915547957939], 1e-6);
+%! assert (randg (0.95, 1, 6), [3.099382433255327 0.3974529788871218 0.644367450750855 1.143261091802246 1.964111762696822 0.04011915547957939], 12*eps);
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! randg ("state", 1);
-%! assert (randg (1, 1, 6), [0.2273389379645993 1.288822625058359 0.2406335209340746 1.218869553370733 1.024649860162554 0.09631230343599533], 1e-6);
+%! assert (randg (1, 1, 6), [0.2273389379645993 1.288822625058359 0.2406335209340746 1.218869553370733 1.024649860162554 0.09631230343599533], 40*eps);
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! randg ("state", 1);
-%! assert (randg (10, 1, 6), [3.520369644331133 15.15369864472106 8.332112081991205 8.406211067432674 11.81193475187611 10.88792728177059], 1e-5);
+%! assert (randg (10, 1, 6), [3.520369644331133 15.15369864472106 8.332112081991205 8.406211067432674 11.81193475187611 10.88792728177059], 56*eps);
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! randg ("state", 1);
-%! assert (randg (100, 1, 6), [75.34570255262264 115.4911985594699 95.23493031356388 95.48926019250911 106.2397448229803 103.4813150404118], 1e-4);
+%! assert (randg (100, 1, 6), [75.34570255262264 115.4911985594699 95.23493031356388 95.48926019250911 106.2397448229803 103.4813150404118], 256*eps);
 %!test
-%! ## Test fixed seed
+%! ## Test a known fixed seed
 %! randg ("seed", 1);
 %! assert (randg (0.1, 1, 6), [0.07144210487604141 0.460641473531723 0.4749028384685516 0.06823389977216721 0.000293838675133884 1.802567535340305e-12], 1e-6);
 %!test
-%! ## Test fixed seed
+%! ## Test a known fixed seed
 %! randg ("seed", 1);
 %! assert (randg (0.95, 1, 6), [1.664905071258545 1.879976987838745 1.905677795410156 0.9948706030845642 0.5606933236122131 0.0766092911362648], 1e-6);
 %!test
-%! ## Test fixed seed
+%! ## Test a known fixed seed
 %! randg ("seed", 1);
 %! assert (randg (1, 1, 6), [0.03512085229158401 0.6488978862762451 0.8114678859710693 0.1666885763406754 1.60791552066803 1.90356981754303], 1e-6);
 %!test
-%! ## Test fixed seed
+%! ## Test a known fixed seed
 %! randg ("seed", 1);
 %! assert (randg (10, 1, 6), [6.566435813903809 10.11648464202881 10.73162078857422 7.747178077697754 6.278522491455078 6.240195751190186], 1e-5);
 %!test
-%! ## Test fixed seed
+%! ## Test a known fixed seed
 %! randg ("seed", 1);
 %! assert (randg (100, 1, 6), [89.40208435058594 101.4734725952148 103.4020004272461 93.62763214111328 88.33104705810547 88.1871337890625], 1e-4);
 %!test
@@ -803,7 +850,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("state", 12);
 %!   a = 0.1;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.01);
 %!   assert (var (x),      a,          0.01);
 %!   assert (skewness (x), 2/sqrt (a), 1);
@@ -814,7 +861,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("state", 12);
 %!   a = 0.95;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.01);
 %!   assert (var (x),      a,          0.04);
 %!   assert (skewness (x), 2/sqrt (a), 0.2);
@@ -825,7 +872,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("state", 12);
 %!   a = 1;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.01);
 %!   assert (var (x),      a,          0.04);
 %!   assert (skewness (x), 2/sqrt (a), 0.2);
@@ -836,7 +883,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("state", 12);
 %!   a = 10;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.1);
 %!   assert (var (x),      a,          0.5);
 %!   assert (skewness (x), 2/sqrt (a), 0.1);
@@ -847,7 +894,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("state", 12);
 %!   a = 100;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.2);
 %!   assert (var (x),      a,          2);
 %!   assert (skewness (x), 2/sqrt (a), 0.05);
@@ -861,7 +908,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("seed", 12);
 %!   a = 0.1;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.01);
 %!   assert (var (x),      a,          0.01);
 %!   assert (skewness (x), 2/sqrt (a), 1);
@@ -872,7 +919,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("seed", 12);
 %!   a = 0.95;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.01);
 %!   assert (var (x),      a,          0.04);
 %!   assert (skewness (x), 2/sqrt (a), 0.2);
@@ -883,7 +930,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("seed", 12);
 %!   a = 1;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.01);
 %!   assert (var (x),      a,          0.04);
 %!   assert (skewness (x), 2/sqrt (a), 0.2);
@@ -894,7 +941,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("seed", 12);
 %!   a = 10;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.1);
 %!   assert (var (x),      a,          0.5);
 %!   assert (skewness (x), 2/sqrt (a), 0.1);
@@ -905,7 +952,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randg ("seed", 12);
 %!   a = 100;
-%!   x = randg (a, 100000, 1);
+%!   x = randg (a, 100_000, 1);
 %!   assert (mean (x),     a,          0.2);
 %!   assert (var (x),      a,          2);
 %!   assert (skewness (x), 2/sqrt (a), 0.05);
@@ -977,28 +1024,28 @@ classes.
 %! randp ("state", 12);
 %! assert (randp ([-inf, -1, 0, inf, nan]), [nan, nan, 0, nan, nan]);   # *** Please report
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! randp ("state", 1);
 %! assert (randp (5, 1, 6), [5 5 3 7 7 3]);
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! randp ("state", 1);
 %! assert (randp (15, 1, 6), [13 15 8 18 18 15]);
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed state
 %! randp ("state", 1);
 %! assert (randp (1e9, 1, 6), [999915677 999976657 1000047684 1000019035 999985749 999977692], -1e-6);
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed seed
 %! randp ("seed", 1);
 %! %%assert (randp (5, 1, 6), [8 2 3 6 6 8])
 %! assert (randp (5, 1, 5), [8 2 3 6 6]);
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed seed
 %! randp ("seed", 1);
 %! assert (randp (15, 1, 6), [15 16 12 10 10 12]);
 %!test
-%! ## Test fixed state
+%! ## Test a known fixed seed
 %! randp ("seed", 1);
 %! assert (randp (1e9, 1, 6), [1000006208 1000012224 999981120 999963520 999963072 999981440], -1e-6);
 %!test
@@ -1006,7 +1053,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randp ("state", 12);
 %!   for a = [5, 15, 1e9; 0.03, 0.03, -5e-3; 0.03, 0.03, 0.03]
-%!     x = randp (a (1), 100000, 1);
+%!     x = randp (a (1), 100_000, 1);
 %!     assert (min (x) >= 0);   # *** Please report this!!! ***
 %!     assert (mean (x), a(1), a(2));
 %!     assert (var (x), a(1), 0.02*a(1));
@@ -1019,7 +1066,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randp ("state", 12);
 %!   for a = [5, 15, 1e9; 0.03, 0.03, -5e-3; 0.03, 0.03, 0.03]
-%!     x = randp (a(1)*ones (100000, 1), 100000, 1);
+%!     x = randp (a(1)*ones (100_000, 1), 100_000, 1);
 %!     assert (min (x) >= 0);   # *** Please report this!!! ***
 %!     assert (mean (x), a(1), a(2));
 %!     assert (var (x), a(1), 0.02*a(1));
@@ -1035,7 +1082,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randp ("seed", 12);
 %!   for a = [5, 15, 1e9; 0.03, 0.03, -5e-3; 0.03, 0.03, 0.03]
-%!     x = randp (a(1), 100000, 1);
+%!     x = randp (a(1), 100_000, 1);
 %!     assert (min (x) >= 0);   # *** Please report this!!! ***
 %!     assert (mean (x), a(1), a(2));
 %!     assert (var (x), a(1), 0.02*a(1));
@@ -1048,7 +1095,7 @@ classes.
 %!   ## statistical tests may fail occasionally.
 %!   randp ("seed", 12);
 %!   for a = [5, 15, 1e9; 0.03, 0.03, -5e-3; 0.03, 0.03, 0.03]
-%!     x = randp (a(1)*ones (100000, 1), 100000, 1);
+%!     x = randp (a(1)*ones (100_000, 1), 100_000, 1);
 %!     assert (min (x) >= 0);   # *** Please report this!!! ***
 %!     assert (mean (x), a(1), a(2));
 %!     assert (var (x), a(1), 0.02*a(1));
@@ -1093,10 +1140,10 @@ likely.
   bool short_shuffle = m < n/5;
 
   // Generate random numbers.
-  NDArray r = octave_rand::nd_array (dim_vector (1, m));
+  NDArray r = rand::nd_array (dim_vector (1, m));
   double *rvec = r.fortran_vec ();
 
-  octave_idx_type idx_len = short_shuffle ? m : n;
+  octave_idx_type idx_len = (short_shuffle ? m : n);
   Array<octave_idx_type> idx;
   try
     {
@@ -1166,7 +1213,7 @@ likely.
 %!assert (length (randperm (20,10)), 10)
 
 ## Test biggish N
-%!assert <39378> (length (randperm (30000^2, 100000)), 100000)
+%!assert <*39378> (length (randperm (30_000^2, 100_000)), 100_000)
 
 %!test
 %! rand ("seed", 0);
@@ -1175,3 +1222,5 @@ likely.
 %!   assert (length (unique (p)), 30);
 %! endfor
 */
+
+OCTAVE_NAMESPACE_END
