@@ -1,25 +1,27 @@
-/*
-
-Copyright (C) 1996-2017 John W. Eaton
-Copyright (C) 2009-2010 VZLU Prague
-
-This file is part of Octave.
-
-Octave is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 3 of the License, or (at your
-option) any later version.
-
-Octave is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2022 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if ! defined (octave_ov_base_h)
 #define octave_ov_base_h 1
@@ -30,6 +32,7 @@ along with Octave; see the file COPYING.  If not, see
 
 #include <iosfwd>
 #include <list>
+#include <memory>
 #include <string>
 
 #include "Range.h"
@@ -37,26 +40,35 @@ along with Octave; see the file COPYING.  If not, see
 #include "mx-base.h"
 #include "str-vec.h"
 
-#include "error.h"
 #include "oct-hdf5-types.h"
+#include "oct-stream.h"
+
+namespace octave
+{
+  class stack_frame;
+  class type_info;
+
+  // FIXME: This is not ideal, but it avoids including
+  // interpreter-private.h here and bringing in a lot of unnecessary
+  // symbols that require even more header files.
+
+  extern OCTINTERP_API type_info& __get_type_info__ (const std::string&);
+}
 
 class Cell;
+class float_display_format;
 class mxArray;
 class octave_map;
 class octave_scalar_map;
 class octave_value;
 class octave_value_list;
-class octave_stream;
+class octave_classdef;
 class octave_function;
 class octave_user_function;
 class octave_user_script;
 class octave_user_code;
 class octave_fcn_handle;
-class octave_fcn_inline;
 class octave_value_list;
-class octave_lvalue;
-
-class tree_walker;
 
 enum builtin_type_t
 {
@@ -81,11 +93,7 @@ enum builtin_type_t
   btyp_num_types = btyp_unknown
 };
 
-extern OCTINTERP_API std::string
-btyp_class_name [btyp_num_types];
-
-extern OCTINTERP_API string_vector
-get_builtin_classes (void);
+extern OCTINTERP_API std::string btyp_class_name [];
 
 inline bool btyp_isnumeric (builtin_type_t btyp)
 { return btyp <= btyp_uint64; }
@@ -99,15 +107,18 @@ inline bool btyp_isfloat (builtin_type_t btyp)
 inline bool btyp_isarray (builtin_type_t btyp)
 { return btyp <= btyp_char; }
 
-// Compute numeric type for a possible mixed-type operation, using these rules:
-// bool -> double
-// single + double -> single
-// real + complex -> complex
-// integer + real -> integer
-// uint + uint -> uint (the bigger one)
-// sint + sint -> sint (the bigger one)
-//
-// failing otherwise.
+//! Determine the resulting type for a possible mixed-type operation.
+//!
+//! Rules for the resulting type:
+//!   - bool -> double
+//!   - single + double -> single
+//!   - real + complex -> complex
+//!   - integer + real -> integer
+//!   - uint + uint -> uint (the bigger one)
+//!   - sint + sint -> sint (the bigger one)
+//!
+//! @return The resulting type or "unknown type", if the resulting type cannot
+//!         be determined.
 
 extern OCTINTERP_API
 builtin_type_t btyp_mixed_numeric (builtin_type_t x, builtin_type_t y);
@@ -118,27 +129,40 @@ struct class_to_btyp
   static const builtin_type_t btyp = btyp_unknown;
 };
 
-#define DEF_CLASS_TO_BTYP(CLASS,BTYP)           \
+template <builtin_type_t BTYP>
+struct btyp_to_class
+{
+  typedef void type;
+};
+
+#define DEF_BTYP_TRAITS(BTYP, CLASS)            \
   template <>                                   \
   struct class_to_btyp<CLASS>                   \
   {                                             \
     static const builtin_type_t btyp = BTYP;    \
+  };                                            \
+                                                \
+  template <>                                   \
+  struct btyp_to_class<BTYP>                    \
+  {                                             \
+    typedef CLASS type;                         \
   }
 
-DEF_CLASS_TO_BTYP (double, btyp_double);
-DEF_CLASS_TO_BTYP (float, btyp_float);
-DEF_CLASS_TO_BTYP (Complex, btyp_complex);
-DEF_CLASS_TO_BTYP (FloatComplex, btyp_float_complex);
-DEF_CLASS_TO_BTYP (octave_int8, btyp_int8);
-DEF_CLASS_TO_BTYP (octave_int16, btyp_int16);
-DEF_CLASS_TO_BTYP (octave_int32, btyp_int32);
-DEF_CLASS_TO_BTYP (octave_int64, btyp_int64);
-DEF_CLASS_TO_BTYP (octave_uint8, btyp_uint8);
-DEF_CLASS_TO_BTYP (octave_uint16, btyp_uint16);
-DEF_CLASS_TO_BTYP (octave_uint32, btyp_uint32);
-DEF_CLASS_TO_BTYP (octave_uint64, btyp_uint64);
-DEF_CLASS_TO_BTYP (bool, btyp_bool);
-DEF_CLASS_TO_BTYP (char, btyp_char);
+DEF_BTYP_TRAITS (btyp_double, double);
+DEF_BTYP_TRAITS (btyp_float, float);
+DEF_BTYP_TRAITS (btyp_complex, Complex);
+DEF_BTYP_TRAITS (btyp_float_complex, FloatComplex);
+DEF_BTYP_TRAITS (btyp_int8, octave_int8);
+DEF_BTYP_TRAITS (btyp_int16, octave_int16);
+DEF_BTYP_TRAITS (btyp_int32, octave_int32);
+DEF_BTYP_TRAITS (btyp_int64, octave_int64);
+DEF_BTYP_TRAITS (btyp_uint8, octave_uint8);
+DEF_BTYP_TRAITS (btyp_uint16, octave_uint16);
+DEF_BTYP_TRAITS (btyp_uint32, octave_uint32);
+DEF_BTYP_TRAITS (btyp_uint64, octave_uint64);
+DEF_BTYP_TRAITS (btyp_bool, bool);
+DEF_BTYP_TRAITS (btyp_char, char);
+
 
 // T_ID is the type id of struct objects, set by register_type().
 // T_NAME is the type name of struct objects.
@@ -160,23 +184,43 @@ DEF_CLASS_TO_BTYP (char, btyp_char);
     static std::string static_type_name (void) { return t_name; }       \
     static std::string static_class_name (void) { return c_name; }      \
     static void register_type (void);                                   \
+    static void register_type (octave::type_info&);                     \
                                                                         \
   private:                                                              \
     static int t_id;                                                    \
     static const std::string t_name;                                    \
     static const std::string c_name;
 
-#define DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA(t, n, c)                    \
-  int t::t_id (-1);                                                     \
-  const std::string t::t_name (n);                                      \
-  const std::string t::c_name (c);                                      \
-  void t::register_type (void)                                          \
+#define DECLARE_TEMPLATE_OV_TYPEID_SPECIALIZATIONS(cls, type)           \
+  template <> void cls<type>::register_type (void);                     \
+  template <> void cls<type>::register_type (octave::type_info&);       \
+  template <> int cls<type>::t_id;                                      \
+  template <> const std::string cls<type>::t_name;                      \
+  template <> const std::string cls<type>::c_name;
+
+#define DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA_INTERNAL(tspec, t, n, c)    \
+  tspec int t::t_id (-1);                                               \
+  tspec const std::string t::t_name (n);                                \
+  tspec const std::string t::c_name (c);                                \
+  tspec void t::register_type (void)                                    \
   {                                                                     \
-    static t exemplar;                                                  \
-    octave_value v (&exemplar, true);                                   \
-    t_id = octave_value_typeinfo::register_type (t::t_name, t::c_name, v); \
+    octave::type_info& type_info                                        \
+      = octave::__get_type_info__ (#t "::register_type");               \
+                                                                        \
+    register_type (type_info);                                          \
+  }                                                                     \
+  tspec void t::register_type (octave::type_info& ti)                   \
+  {                                                                     \
+    octave_value v (new t ());                                          \
+    t_id = ti.register_type (t::t_name, t::c_name, v);                  \
   }
 
+#define DEFINE_TEMPLATE_OV_TYPEID_FUNCTIONS_AND_DATA(t, n, c)           \
+  DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA_INTERNAL (template <>, t, n, c)
+
+#define DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA(t, n, c)            \
+  DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA_INTERNAL ( , t, n, c)
+
 // A base value type, so that derived types only have to redefine what
 // they need (if they are derived from octave_base_value instead of
 // octave_value).
@@ -193,19 +237,19 @@ public:
   class type_conv_info
   {
   public:
-    type_conv_info (type_conv_fcn f = 0, int t = -1)
-      : _fcn (f), _type_id (t) { }
+    type_conv_info (type_conv_fcn f = nullptr, int t = -1)
+      : m_fcn (f), m_type_id (t) { }
 
-    operator type_conv_fcn (void) const { return _fcn; }
+    operator type_conv_fcn (void) const { return m_fcn; }
 
-    octave_base_value * operator () (const octave_base_value &v) const
-    { return (*_fcn) (v); }
+    octave_base_value * operator () (const octave_base_value& v) const
+    { return (*m_fcn) (v); }
 
-    int type_id (void) const { return _type_id; }
+    int type_id (void) const { return m_type_id; }
 
   private:
-    type_conv_fcn _fcn;
-    int _type_id;
+    type_conv_fcn m_fcn;
+    int m_type_id;
   };
 
   friend class octave_value;
@@ -214,7 +258,7 @@ public:
 
   octave_base_value (const octave_base_value&) : count (1) { }
 
-  virtual ~octave_base_value (void) { }
+  virtual ~octave_base_value (void) = default;
 
   // Unconditional clone.  Always clones.
   virtual octave_base_value *
@@ -230,6 +274,8 @@ public:
   virtual octave_base_value *
   unique_clone (void) { return clone (); }
 
+  virtual void break_closure_cycles (const std::shared_ptr<octave::stack_frame>&) { }
+
   virtual type_conv_info
   numeric_conversion_function (void) const
   { return type_conv_info (); }
@@ -255,13 +301,15 @@ public:
   virtual octave_value as_uint32 (void) const;
   virtual octave_value as_uint64 (void) const;
 
-  virtual octave_base_value *try_narrowing_conversion (void) { return 0; }
+  virtual octave_base_value * try_narrowing_conversion (void) { return nullptr; }
 
   virtual void maybe_economize (void) { }
 
   virtual Matrix size (void);
 
-  virtual octave_idx_type numel (const octave_value_list&);
+  virtual octave_idx_type xnumel (const octave_value_list&);
+
+  // FIXME: Do we really need all three of these versions of subsref?
 
   virtual octave_value
   subsref (const std::string& type,
@@ -277,22 +325,9 @@ public:
            const std::list<octave_value_list>& idx,
            bool auto_add);
 
-  virtual octave_value_list
-  subsref (const std::string& type,
-           const std::list<octave_value_list>& idx,
-           int nargout,
-           const std::list<octave_lvalue> *lvalue_list);
-
   virtual octave_value
   do_index_op (const octave_value_list& idx, bool resize_ok = false);
 
-  virtual octave_value_list
-  do_multi_index_op (int nargout, const octave_value_list& idx);
-
-  virtual octave_value_list
-  do_multi_index_op (int nargout, const octave_value_list& idx,
-                     const std::list<octave_lvalue> *lvalue_list);
-
   virtual void assign (const std::string&, const octave_value&) { }
 
   virtual octave_value
@@ -305,7 +340,7 @@ public:
                   const std::list<octave_value_list>& idx,
                   const octave_value& rhs);
 
-  virtual idx_vector index_vector (bool require_integers = false) const;
+  virtual octave::idx_vector index_vector (bool require_integers = false) const;
 
   virtual dim_vector dims (void) const { return dim_vector (); }
 
@@ -328,11 +363,7 @@ public:
 
   virtual octave_idx_type numel (void) const { return dims ().numel (); }
 
-  OCTAVE_DEPRECATED ("use 'numel' instead")
-  virtual octave_idx_type capacity (void) const
-  { return numel (); }
-
-  virtual size_t byte_size (void) const { return 0; }
+  virtual std::size_t byte_size (void) const { return 0; }
 
   virtual octave_idx_type nnz (void) const;
 
@@ -352,11 +383,15 @@ public:
 
   virtual bool is_defined (void) const { return false; }
 
-  bool is_empty (void) const { return (dims ().any_zero ()); }
+  virtual bool is_storable (void) const { return true; }
 
-  virtual bool is_cell (void) const { return false; }
+  bool isempty (void) const { return (dims ().any_zero ()); }
 
-  virtual bool is_cellstr (void) const { return false; }
+  bool is_zero_by_zero (void) const { return dims().zero_by_zero (); }
+
+  virtual bool iscell (void) const { return false; }
+
+  virtual bool iscellstr (void) const { return false; }
 
   virtual bool is_real_scalar (void) const { return false; }
 
@@ -382,13 +417,19 @@ public:
 
   virtual bool is_range (void) const { return false; }
 
-  virtual bool is_map (void) const { return false; }
+  virtual bool isstruct (void) const { return false; }
 
-  virtual bool is_object (void) const { return false; }
+  virtual bool isobject (void) const { return false; }
+
+  virtual bool is_classdef_meta (void) const { return false; }
+
+  virtual bool is_classdef_superclass_ref (void) const { return false; }
 
   virtual bool is_classdef_object (void) const { return false; }
 
-  virtual bool is_java (void) const { return false; }
+  virtual bool is_package (void) const { return false; }
+
+  virtual bool isjava (void) const { return false; }
 
   virtual bool is_cs_list (void) const { return false; }
 
@@ -406,7 +447,7 @@ public:
 
   virtual bool is_single_type (void) const { return false; }
 
-  virtual bool is_float_type (void) const { return false; }
+  virtual bool isfloat (void) const { return false; }
 
   virtual bool is_int8_type (void) const { return false; }
 
@@ -424,13 +465,13 @@ public:
 
   virtual bool is_uint64_type (void) const { return false; }
 
-  virtual bool is_bool_type (void) const { return false; }
+  virtual bool islogical (void) const { return false; }
 
-  virtual bool is_integer_type (void) const { return false; }
+  virtual bool isinteger (void) const { return false; }
 
-  virtual bool is_real_type (void) const { return false; }
+  virtual bool isreal (void) const { return false; }
 
-  virtual bool is_complex_type (void) const { return false; }
+  virtual bool iscomplex (void) const { return false; }
 
   // Would be nice to get rid of the next four functions:
 
@@ -438,13 +479,15 @@ public:
 
   virtual bool is_matrix_type (void) const { return false; }
 
-  virtual bool is_numeric_type (void) const { return false; }
+  virtual bool isnumeric (void) const { return false; }
 
-  virtual bool is_sparse_type (void) const { return false; }
+  virtual bool issparse (void) const { return false; }
 
   virtual bool is_true (void) const { return false; }
 
-  virtual bool is_null_value (void) const { return false; }
+  virtual bool is_magic_int (void) const { return false; }
+
+  virtual bool isnull (void) const { return false; }
 
   virtual bool is_constant (void) const { return false; }
 
@@ -585,7 +628,25 @@ public:
 
   virtual Array<std::string> cellstr_value (void) const;
 
-  virtual Range range_value (void) const;
+  virtual octave::range<float> float_range_value (void) const;
+
+  virtual octave::range<double> range_value (void) const;
+
+  virtual octave::range<octave_int8> int8_range_value (void) const;
+
+  virtual octave::range<octave_int16> int16_range_value (void) const;
+
+  virtual octave::range<octave_int32> int32_range_value (void) const;
+
+  virtual octave::range<octave_int64> int64_range_value (void) const;
+
+  virtual octave::range<octave_uint8> uint8_range_value (void) const;
+
+  virtual octave::range<octave_uint16> uint16_range_value (void) const;
+
+  virtual octave::range<octave_uint32> uint32_range_value (void) const;
+
+  virtual octave::range<octave_uint64> uint64_range_value (void) const;
 
   virtual octave_map map_value (void) const;
 
@@ -593,32 +654,32 @@ public:
 
   virtual string_vector map_keys (void) const;
 
-  virtual size_t nparents (void) const;
+  virtual std::size_t nparents (void) const;
 
   virtual std::list<std::string> parent_class_name_list (void) const;
 
   virtual string_vector parent_class_names (void) const;
 
-  virtual octave_base_value *find_parent_class (const std::string&)
-  { return 0; }
+  virtual octave_base_value * find_parent_class (const std::string&)
+  { return nullptr; }
 
-  virtual octave_base_value *unique_parent_class (const std::string&)
-  { return 0; }
+  virtual octave_base_value * unique_parent_class (const std::string&)
+  { return nullptr; }
 
   virtual bool is_instance_of (const std::string&) const
   { return false; }
 
-  virtual octave_function *function_value (bool silent = false);
+  virtual octave_classdef * classdef_object_value (bool silent = false);
 
-  virtual octave_user_function *user_function_value (bool silent = false);
+  virtual octave_function * function_value (bool silent = false);
 
-  virtual octave_user_script *user_script_value (bool silent = false);
+  virtual octave_user_function * user_function_value (bool silent = false);
 
-  virtual octave_user_code *user_code_value (bool silent = false);
+  virtual octave_user_script * user_script_value (bool silent = false);
 
-  virtual octave_fcn_handle *fcn_handle_value (bool silent = false);
+  virtual octave_user_code * user_code_value (bool silent = false);
 
-  virtual octave_fcn_inline *fcn_inline_value (bool silent = false);
+  virtual octave_fcn_handle * fcn_handle_value (bool silent = false);
 
   virtual octave_value_list list_value (void) const;
 
@@ -650,13 +711,19 @@ public:
 
   virtual void short_disp (std::ostream& os) const { os << "..."; }
 
+  virtual float_display_format get_edit_display_format (void) const;
+
+  virtual std::string edit_display (const float_display_format&,
+                                    octave_idx_type, octave_idx_type) const
+  { return "#VAL"; }
+
   virtual void print_info (std::ostream& os, const std::string& prefix) const;
 
   virtual bool save_ascii (std::ostream& os);
 
   virtual bool load_ascii (std::istream& is);
 
-  virtual bool save_binary (std::ostream& os, bool& save_as_floats);
+  virtual bool save_binary (std::ostream& os, bool save_as_floats);
 
   virtual bool load_binary (std::istream& is, bool swap,
                             octave::mach_info::float_format fmt);
@@ -668,17 +735,17 @@ public:
   load_hdf5 (octave_hdf5_id loc_id, const char *name);
 
   virtual int
-  write (octave_stream& os, int block_size,
+  write (octave::stream& os, int block_size,
          oct_data_conv::data_type output_type, int skip,
          octave::mach_info::float_format flt_fmt) const;
 
-  virtual void *mex_get_data (void) const { return 0; }
+  virtual const void * mex_get_data (void) const { return nullptr; }
 
-  virtual octave_idx_type *mex_get_ir (void) const { return 0; }
+  virtual const octave_idx_type * mex_get_ir (void) const { return nullptr; }
 
-  virtual octave_idx_type *mex_get_jc (void) const { return 0; }
+  virtual const octave_idx_type * mex_get_jc (void) const { return nullptr; }
 
-  virtual mxArray *as_mxArray (void) const;
+  virtual mxArray * as_mxArray (bool interleaved) const;
 
   virtual octave_value diag (octave_idx_type k = 0) const;
 
@@ -686,11 +753,11 @@ public:
 
   virtual octave_value sort (octave_idx_type dim = 0,
                              sortmode mode = ASCENDING) const;
-  virtual octave_value sort (Array<octave_idx_type> &sidx,
+  virtual octave_value sort (Array<octave_idx_type>& sidx,
                              octave_idx_type dim = 0,
                              sortmode mode = ASCENDING) const;
 
-  virtual sortmode is_sorted (sortmode mode = UNSORTED) const;
+  virtual sortmode issorted (sortmode mode = UNSORTED) const;
 
   virtual Array<octave_idx_type>
   sort_rows_idx (sortmode mode = ASCENDING) const;
@@ -703,7 +770,9 @@ public:
 
   virtual bool islocked (void) const { return false; }
 
-  virtual void dump (std::ostream& os) const;
+  virtual void call_object_destructor (void) { }
+
+  virtual octave_value dump (void) const;
 
   // Standard mappers.  Register new ones here.
   enum unary_mapper_t
@@ -766,7 +835,6 @@ public:
     umap_xisupper,
     umap_xisxdigit,
     umap_xsignbit,
-    umap_xtoascii,
     umap_xtolower,
     umap_xtoupper,
     umap_unknown,
@@ -795,64 +863,52 @@ public:
   virtual bool
   fast_elem_insert_self (void *where, builtin_type_t btyp) const;
 
-  // Grab the reference count.  For use by jit.
-  void
-  grab (void)
-  {
-    ++count;
-  }
-
-  // Release the reference count.  For use by jit.
-  void
-  release (void)
-  {
-    if (--count == 0)
-      delete this;
-  }
-
 protected:
 
   // This should only be called for derived types.
 
-  octave_value numeric_assign (const std::string& type,
-                               const std::list<octave_value_list>& idx,
-                               const octave_value& rhs);
+  OCTINTERP_API octave_value
+  numeric_assign (const std::string& type,
+                  const std::list<octave_value_list>& idx,
+                  const octave_value& rhs);
 
   void reset_indent_level (void) const
-  { curr_print_indent_level = 0; }
+  { s_curr_print_indent_level = 0; }
 
   void increment_indent_level (void) const
-  { curr_print_indent_level += 2; }
+  { s_curr_print_indent_level += 2; }
 
   void decrement_indent_level (void) const
-  { curr_print_indent_level -= 2; }
+  { s_curr_print_indent_level -= 2; }
 
   int current_print_indent_level (void) const
-  { return curr_print_indent_level; }
+  { return s_curr_print_indent_level; }
 
-  void indent (std::ostream& os) const;
+  OCTINTERP_API void indent (std::ostream& os) const;
 
-  void newline (std::ostream& os) const;
+  OCTINTERP_API void newline (std::ostream& os) const;
 
-  void reset (void) const;
+  OCTINTERP_API void reset (void) const;
 
   // A reference count.
   // NOTE: the declaration is octave_idx_type because with 64-bit indexing,
   // it is well possible to have more than MAX_INT copies of a single value
   // (think of an empty cell array with >2G elements).
-  octave_refcount<octave_idx_type> count;
+  octave::refcount<octave_idx_type> count;
 
-  static const char *get_umap_name (unary_mapper_t);
+  OCTINTERP_API static const char * get_umap_name (unary_mapper_t);
 
-  void warn_load (const char *type) const;
-  void warn_save (const char *type) const;
+  OCTINTERP_API void warn_load (const char *type) const;
+  OCTINTERP_API void warn_save (const char *type) const;
 
 private:
 
-  void wrong_type_arg_error (void) const;
+  OCTINTERP_API void wrong_type_arg_error (void) const;
 
-  static int curr_print_indent_level;
-  static bool beginning_of_line;
+  //--------
+
+  static int s_curr_print_indent_level;
+  static bool s_beginning_of_line;
 
   DECLARE_OV_BASE_TYPEID_FUNCTIONS_AND_DATA
 };
