@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1998-2021 The Octave Project Developers
+// Copyright (C) 1998-2022 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -46,7 +46,6 @@
 #include "oct-locbuf.h"
 
 #include "Sparse.h"
-#include "sparse-sort.h"
 #include "sparse-util.h"
 #include "oct-spparms.h"
 #include "mx-inlines.cc"
@@ -54,7 +53,7 @@
 #include "PermMatrix.h"
 
 template <typename T>
-typename Sparse<T>::SparseRep *
+OCTAVE_API typename Sparse<T>::SparseRep *
 Sparse<T>::nil_rep (void)
 {
   static typename Sparse<T>::SparseRep nr;
@@ -62,58 +61,61 @@ Sparse<T>::nil_rep (void)
 }
 
 template <typename T>
+OCTAVE_API
 T&
-Sparse<T>::SparseRep::elem (octave_idx_type _r, octave_idx_type _c)
+Sparse<T>::SparseRep::elem (octave_idx_type r, octave_idx_type c)
 {
   octave_idx_type i;
 
-  if (nzmx <= 0)
+  if (m_nzmax <= 0)
     (*current_liboctave_error_handler)
       ("Sparse::SparseRep::elem (octave_idx_type, octave_idx_type): sparse matrix filled");
 
-  for (i = c[_c]; i < c[_c + 1]; i++)
-    if (r[i] == _r)
-      return d[i];
-    else if (r[i] > _r)
+  for (i = m_cidx[c]; i < m_cidx[c + 1]; i++)
+    if (m_ridx[i] == r)
+      return m_data[i];
+    else if (m_ridx[i] > r)
       break;
 
   // Ok, If we've gotten here, we're in trouble.  Have to create a
   // new element in the sparse array.  This' gonna be slow!!!
-  if (c[ncols] == nzmx)
+  if (m_cidx[m_ncols] == m_nzmax)
     (*current_liboctave_error_handler)
       ("Sparse::SparseRep::elem (octave_idx_type, octave_idx_type): sparse matrix filled");
 
-  octave_idx_type to_move = c[ncols] - i;
+  octave_idx_type to_move = m_cidx[m_ncols] - i;
   if (to_move != 0)
     {
-      for (octave_idx_type j = c[ncols]; j > i; j--)
+      for (octave_idx_type j = m_cidx[m_ncols]; j > i; j--)
         {
-          d[j] = d[j-1];
-          r[j] = r[j-1];
+          m_data[j] = m_data[j-1];
+          m_ridx[j] = m_ridx[j-1];
         }
     }
 
-  for (octave_idx_type j = _c + 1; j < ncols + 1; j++)
-    c[j] = c[j] + 1;
+  for (octave_idx_type j = c + 1; j < m_ncols + 1; j++)
+    m_cidx[j] = m_cidx[j] + 1;
 
-  d[i] = 0.;
-  r[i] = _r;
+  m_data[i] = 0.;
+  m_ridx[i] = r;
 
-  return d[i];
+  return m_data[i];
 }
 
 template <typename T>
+OCTAVE_API
 T
-Sparse<T>::SparseRep::celem (octave_idx_type _r, octave_idx_type _c) const
+Sparse<T>::SparseRep::celem (octave_idx_type r, octave_idx_type c) const
 {
-  if (nzmx > 0)
-    for (octave_idx_type i = c[_c]; i < c[_c + 1]; i++)
-      if (r[i] == _r)
-        return d[i];
+  if (m_nzmax > 0)
+    for (octave_idx_type i = m_cidx[c]; i < m_cidx[c + 1]; i++)
+      if (m_ridx[i] == r)
+        return m_data[i];
   return T ();
 }
 
 template <typename T>
+OCTAVE_API
 void
 Sparse<T>::SparseRep::maybe_compress (bool remove_zeros)
 {
@@ -121,82 +123,86 @@ Sparse<T>::SparseRep::maybe_compress (bool remove_zeros)
     {
       octave_idx_type i = 0;
       octave_idx_type k = 0;
-      for (octave_idx_type j = 1; j <= ncols; j++)
+      for (octave_idx_type j = 1; j <= m_ncols; j++)
         {
-          octave_idx_type u = c[j];
+          octave_idx_type u = m_cidx[j];
           for (; i < u; i++)
-            if (d[i] != T ())
+            if (m_data[i] != T ())
               {
-                d[k] = d[i];
-                r[k++] = r[i];
+                m_data[k] = m_data[i];
+                m_ridx[k++] = m_ridx[i];
               }
-          c[j] = k;
+          m_cidx[j] = k;
         }
     }
 
-  change_length (c[ncols]);
+  change_length (m_cidx[m_ncols]);
 }
 
 template <typename T>
+OCTAVE_API
 void
 Sparse<T>::SparseRep::change_length (octave_idx_type nz)
 {
-  for (octave_idx_type j = ncols; j > 0 && c[j] > nz; j--)
-    c[j] = nz;
+  for (octave_idx_type j = m_ncols; j > 0 && m_cidx[j] > nz; j--)
+    m_cidx[j] = nz;
 
   // Always preserve space for 1 element.
   nz = (nz > 0 ? nz : 1);
 
   // Skip reallocation if we have less than 1/frac extra elements to discard.
   static const int frac = 5;
-  if (nz > nzmx || nz < nzmx - nzmx/frac)
+  if (nz > m_nzmax || nz < m_nzmax - m_nzmax/frac)
     {
       // Reallocate.
-      octave_idx_type min_nzmx = std::min (nz, nzmx);
+      octave_idx_type min_nzmax = std::min (nz, m_nzmax);
 
       octave_idx_type *new_ridx = new octave_idx_type [nz];
-      std::copy_n (r, min_nzmx, new_ridx);
+      std::copy_n (m_ridx, min_nzmax, new_ridx);
 
-      delete [] r;
-      r = new_ridx;
+      delete [] m_ridx;
+      m_ridx = new_ridx;
 
       T *new_data = new T [nz];
-      std::copy_n (d, min_nzmx, new_data);
+      std::copy_n (m_data, min_nzmax, new_data);
 
-      delete [] d;
-      d = new_data;
+      delete [] m_data;
+      m_data = new_data;
 
-      nzmx = nz;
+      m_nzmax = nz;
     }
 }
 
 template <typename T>
+OCTAVE_API
 bool
 Sparse<T>::SparseRep::indices_ok (void) const
 {
-  return sparse_indices_ok (r, c, nrows, ncols, nnz ());
+  return sparse_indices_ok (m_ridx, m_cidx, m_nrows, m_ncols, nnz ());
 }
 
 template <typename T>
+OCTAVE_API
 bool
 Sparse<T>::SparseRep::any_element_is_nan (void) const
 {
   octave_idx_type nz = nnz ();
 
   for (octave_idx_type i = 0; i < nz; i++)
-    if (octave::math::isnan (d[i]))
+    if (octave::math::isnan (m_data[i]))
       return true;
 
   return false;
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>::Sparse (octave_idx_type nr, octave_idx_type nc, T val)
-  : rep (nullptr), dimensions (dim_vector (nr, nc))
+  : m_rep (nullptr), m_dimensions (dim_vector (nr, nc))
 {
   if (val != T ())
     {
-      rep = new typename Sparse<T>::SparseRep (nr, nc, dimensions.safe_numel ());
+      m_rep = new typename Sparse<T>::SparseRep (nr, nc, m_dimensions.safe_numel ());
 
       octave_idx_type ii = 0;
       xcidx (0) = 0;
@@ -212,16 +218,17 @@ Sparse<T>::Sparse (octave_idx_type nr, octave_idx_type nc, T val)
     }
   else
     {
-      rep = new typename Sparse<T>::SparseRep (nr, nc, 0);
+      m_rep = new typename Sparse<T>::SparseRep (nr, nc, 0);
       for (octave_idx_type j = 0; j < nc+1; j++)
         xcidx (j) = 0;
     }
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>::Sparse (const PermMatrix& a)
-  : rep (new typename Sparse<T>::SparseRep (a.rows (), a.cols (), a.rows ())),
-    dimensions (dim_vector (a.rows (), a.cols ()))
+  : m_rep (new typename Sparse<T>::SparseRep (a.rows (), a.cols (), a.rows ())),
+    m_dimensions (dim_vector (a.rows (), a.cols ()))
 {
   octave_idx_type n = a.rows ();
   for (octave_idx_type i = 0; i <= n; i++)
@@ -237,19 +244,21 @@ Sparse<T>::Sparse (const PermMatrix& a)
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>::Sparse (const dim_vector& dv)
-  : rep (nullptr), dimensions (dv)
+  : m_rep (nullptr), m_dimensions (dv)
 {
   if (dv.ndims () != 2)
     (*current_liboctave_error_handler)
       ("Sparse::Sparse (const dim_vector&): dimension mismatch");
 
-  rep = new typename Sparse<T>::SparseRep (dv(0), dv(1), 0);
+  m_rep = new typename Sparse<T>::SparseRep (dv(0), dv(1), 0);
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>::Sparse (const Sparse<T>& a, const dim_vector& dv)
-  : rep (nullptr), dimensions (dv)
+  : m_rep (nullptr), m_dimensions (dv)
 {
 
   // Work in unsigned long long to avoid overflow issues with numel
@@ -263,13 +272,13 @@ Sparse<T>::Sparse (const Sparse<T>& a, const dim_vector& dv)
       ("Sparse::Sparse (const Sparse&, const dim_vector&): dimension mismatch");
 
   dim_vector old_dims = a.dims ();
-  octave_idx_type new_nzmx = a.nnz ();
+  octave_idx_type new_nzmax = a.nnz ();
   octave_idx_type new_nr = dv(0);
   octave_idx_type new_nc = dv(1);
   octave_idx_type old_nr = old_dims(0);
   octave_idx_type old_nc = old_dims(1);
 
-  rep = new typename Sparse<T>::SparseRep (new_nr, new_nc, new_nzmx);
+  m_rep = new typename Sparse<T>::SparseRep (new_nr, new_nc, new_nzmax);
 
   octave_idx_type kk = 0;
   xcidx (0) = 0;
@@ -286,15 +295,16 @@ Sparse<T>::Sparse (const Sparse<T>& a, const dim_vector& dv)
         xridx (j) = ii;
       }
   for (octave_idx_type k = kk; k < new_nc; k++)
-    xcidx (k+1) = new_nzmx;
+    xcidx (k+1) = new_nzmax;
 }
 
 template <typename T>
-Sparse<T>::Sparse (const Array<T>& a, const idx_vector& r,
-                   const idx_vector& c, octave_idx_type nr,
+OCTAVE_API
+Sparse<T>::Sparse (const Array<T>& a, const octave::idx_vector& r,
+                   const octave::idx_vector& c, octave_idx_type nr,
                    octave_idx_type nc, bool sum_terms,
                    octave_idx_type nzm)
-  : rep (nullptr), dimensions ()
+  : m_rep (nullptr), m_dimensions ()
 {
   if (nr < 0)
     nr = r.extent (0);
@@ -310,7 +320,7 @@ Sparse<T>::Sparse (const Array<T>& a, const idx_vector& r,
       ("sparse: column index %" OCTAVE_IDX_TYPE_FORMAT " out of bound "
        "%" OCTAVE_IDX_TYPE_FORMAT, r.extent (nc), nc);
 
-  dimensions = dim_vector (nr, nc);
+  m_dimensions = dim_vector (nr, nc);
 
   octave_idx_type n = a.numel ();
   octave_idx_type rl = r.length (nr);
@@ -327,8 +337,8 @@ Sparse<T>::Sparse (const Array<T>& a, const idx_vector& r,
   if ((rl != 1 && rl != n) || (cl != 1 && cl != n))
     (*current_liboctave_error_handler) ("sparse: dimension mismatch");
 
-  // Only create rep after input validation to avoid memory leak.
-  rep = new typename Sparse<T>::SparseRep (nr, nc, (nzm > 0 ? nzm : 0));
+  // Only create m_rep after input validation to avoid memory leak.
+  m_rep = new typename Sparse<T>::SparseRep (nr, nc, (nzm > 0 ? nzm : 0));
 
   if (rl <= 1 && cl <= 1)
     {
@@ -351,7 +361,7 @@ Sparse<T>::Sparse (const Array<T>& a, const idx_vector& r,
       else if (cl == 1)
         {
           // Sparse column vector.  Sort row indices.
-          idx_vector rs = r.sorted ();
+          octave::idx_vector rs = r.sorted ();
 
           octave_quit ();
 
@@ -405,8 +415,8 @@ Sparse<T>::Sparse (const Array<T>& a, const idx_vector& r,
         }
       else
         {
-          idx_vector rr = r;
-          idx_vector cc = c;
+          octave::idx_vector rr = r;
+          octave::idx_vector cc = c;
           const octave_idx_type *rd = rr.raw ();
           const octave_idx_type *cd = cc.raw ();
           OCTAVE_LOCAL_BUFFER_INIT (octave_idx_type, ci, nc+1, 0);
@@ -499,7 +509,7 @@ Sparse<T>::Sparse (const Array<T>& a, const idx_vector& r,
     {
       // Sparse column vector.  Sort row indices.
       Array<octave_idx_type> rsi;
-      idx_vector rs = r.sorted (rsi);
+      octave::idx_vector rs = r.sorted (rsi);
 
       octave_quit ();
 
@@ -552,8 +562,8 @@ Sparse<T>::Sparse (const Array<T>& a, const idx_vector& r,
     }
   else
     {
-      idx_vector rr = r;
-      idx_vector cc = c;
+      octave::idx_vector rr = r;
+      octave::idx_vector cc = c;
       const octave_idx_type *rd = rr.raw ();
       const octave_idx_type *cd = cc.raw ();
       OCTAVE_LOCAL_BUFFER_INIT (octave_idx_type, ci, nc+1, 0);
@@ -651,39 +661,40 @@ Sparse<T>::Sparse (const Array<T>& a, const idx_vector& r,
 }
 
 /*
-%!assert <51880> (sparse (1:2, 2, 1:2, 2, 2), sparse ([0, 1; 0, 2]))
-%!assert <51880> (sparse (1:2, 1, 1:2, 2, 2), sparse ([1, 0; 2, 0]))
-%!assert <51880> (sparse (1:2, 2, 1:2, 2, 3), sparse ([0, 1, 0; 0, 2, 0]))
+%!assert <*51880> (sparse (1:2, 2, 1:2, 2, 2), sparse ([0, 1; 0, 2]))
+%!assert <*51880> (sparse (1:2, 1, 1:2, 2, 2), sparse ([1, 0; 2, 0]))
+%!assert <*51880> (sparse (1:2, 2, 1:2, 2, 3), sparse ([0, 1, 0; 0, 2, 0]))
 */
 
 template <typename T>
+OCTAVE_API
 Sparse<T>::Sparse (const Array<T>& a)
-  : rep (nullptr), dimensions (a.dims ())
+  : m_rep (nullptr), m_dimensions (a.dims ())
 {
-  if (dimensions.ndims () > 2)
+  if (m_dimensions.ndims () > 2)
     (*current_liboctave_error_handler)
       ("Sparse::Sparse (const Array<T>&): dimension mismatch");
 
   octave_idx_type nr = rows ();
   octave_idx_type nc = cols ();
   octave_idx_type len = a.numel ();
-  octave_idx_type new_nzmx = 0;
+  octave_idx_type new_nzmax = 0;
 
   // First count the number of nonzero terms
   for (octave_idx_type i = 0; i < len; i++)
     if (a(i) != T ())
-      new_nzmx++;
+      new_nzmax++;
 
-  rep = new typename Sparse<T>::SparseRep (nr, nc, new_nzmx);
+  m_rep = new typename Sparse<T>::SparseRep (nr, nc, new_nzmax);
 
   octave_idx_type ii = 0;
   xcidx (0) = 0;
   for (octave_idx_type j = 0; j < nc; j++)
     {
       for (octave_idx_type i = 0; i < nr; i++)
-        if (a.elem (i,j) != T ())
+        if (a.elem (i, j) != T ())
           {
-            xdata (ii) = a.elem (i,j);
+            xdata (ii) = a.elem (i, j);
             xridx (ii++) = i;
           }
       xcidx (j+1) = ii;
@@ -691,10 +702,11 @@ Sparse<T>::Sparse (const Array<T>& a)
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>::~Sparse (void)
 {
-  if (--rep->count == 0)
-    delete rep;
+  if (--m_rep->m_count == 0)
+    delete m_rep;
 }
 
 template <typename T>
@@ -703,23 +715,24 @@ Sparse<T>::operator = (const Sparse<T>& a)
 {
   if (this != &a)
     {
-      if (--rep->count == 0)
-        delete rep;
+      if (--m_rep->m_count == 0)
+        delete m_rep;
 
-      rep = a.rep;
-      rep->count++;
+      m_rep = a.m_rep;
+      m_rep->m_count++;
 
-      dimensions = a.dimensions;
+      m_dimensions = a.m_dimensions;
     }
 
   return *this;
 }
 
 template <typename T>
+OCTAVE_API
 octave_idx_type
 Sparse<T>::compute_index (const Array<octave_idx_type>& ra_idx) const
 {
-  octave_idx_type n = dimensions.ndims ();
+  octave_idx_type n = m_dimensions.ndims ();
 
   if (n <= 0 || n != ra_idx.numel ())
     (*current_liboctave_error_handler)
@@ -731,7 +744,7 @@ Sparse<T>::compute_index (const Array<octave_idx_type>& ra_idx) const
 
   while (--n >= 0)
     {
-      retval *= dimensions(n);
+      retval *= m_dimensions(n);
       retval += ra_idx(n);
     }
 
@@ -739,6 +752,7 @@ Sparse<T>::compute_index (const Array<octave_idx_type>& ra_idx) const
 }
 
 template <typename T>
+OCTAVE_API
 T
 Sparse<T>::range_error (const char *fcn, octave_idx_type n) const
 {
@@ -747,6 +761,7 @@ Sparse<T>::range_error (const char *fcn, octave_idx_type n) const
 }
 
 template <typename T>
+OCTAVE_API
 T&
 Sparse<T>::range_error (const char *fcn, octave_idx_type n)
 {
@@ -755,6 +770,7 @@ Sparse<T>::range_error (const char *fcn, octave_idx_type n)
 }
 
 template <typename T>
+OCTAVE_API
 T
 Sparse<T>::range_error (const char *fcn, octave_idx_type i,
                         octave_idx_type j) const
@@ -765,6 +781,7 @@ Sparse<T>::range_error (const char *fcn, octave_idx_type i,
 }
 
 template <typename T>
+OCTAVE_API
 T&
 Sparse<T>::range_error (const char *fcn, octave_idx_type i, octave_idx_type j)
 {
@@ -774,6 +791,7 @@ Sparse<T>::range_error (const char *fcn, octave_idx_type i, octave_idx_type j)
 }
 
 template <typename T>
+OCTAVE_API
 T
 Sparse<T>::range_error (const char *fcn,
                         const Array<octave_idx_type>& ra_idx) const
@@ -798,6 +816,7 @@ Sparse<T>::range_error (const char *fcn,
 }
 
 template <typename T>
+OCTAVE_API
 T&
 Sparse<T>::range_error (const char *fcn, const Array<octave_idx_type>& ra_idx)
 {
@@ -821,6 +840,7 @@ Sparse<T>::range_error (const char *fcn, const Array<octave_idx_type>& ra_idx)
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>
 Sparse<T>::reshape (const dim_vector& new_dims) const
 {
@@ -839,9 +859,9 @@ Sparse<T>::reshape (const dim_vector& new_dims) const
       dims2.resize (2);
     }
 
-  if (dimensions != dims2)
+  if (m_dimensions != dims2)
     {
-      if (dimensions.numel () == dims2.numel ())
+      if (m_dimensions.numel () == dims2.numel ())
         {
           octave_idx_type new_nnz = nnz ();
           octave_idx_type new_nr = dims2 (0);
@@ -885,7 +905,7 @@ Sparse<T>::reshape (const dim_vector& new_dims) const
         }
       else
         {
-          std::string dimensions_str = dimensions.str ();
+          std::string dimensions_str = m_dimensions.str ();
           std::string new_dims_str = new_dims.str ();
 
           (*current_liboctave_error_handler)
@@ -900,6 +920,7 @@ Sparse<T>::reshape (const dim_vector& new_dims) const
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>
 Sparse<T>::permute (const Array<octave_idx_type>& perm_vec, bool) const
 {
@@ -928,6 +949,7 @@ Sparse<T>::permute (const Array<octave_idx_type>& perm_vec, bool) const
 }
 
 template <typename T>
+OCTAVE_API
 void
 Sparse<T>::resize1 (octave_idx_type n)
 {
@@ -947,6 +969,7 @@ Sparse<T>::resize1 (octave_idx_type n)
 }
 
 template <typename T>
+OCTAVE_API
 void
 Sparse<T>::resize (const dim_vector& dv)
 {
@@ -959,6 +982,7 @@ Sparse<T>::resize (const dim_vector& dv)
 }
 
 template <typename T>
+OCTAVE_API
 void
 Sparse<T>::resize (octave_idx_type r, octave_idx_type c)
 {
@@ -968,7 +992,7 @@ Sparse<T>::resize (octave_idx_type r, octave_idx_type c)
   if (r == dim1 () && c == dim2 ())
     return;
 
-  // This wouldn't be necessary for r >= rows () if nrows wasn't part of the
+  // This wouldn't be necessary for r >= rows () if m_nrows wasn't part of the
   // Sparse rep.  It is not good for anything in there.
   make_unique ();
 
@@ -976,7 +1000,7 @@ Sparse<T>::resize (octave_idx_type r, octave_idx_type c)
     {
       octave_idx_type i = 0;
       octave_idx_type k = 0;
-      for (octave_idx_type j = 1; j <= rep->ncols; j++)
+      for (octave_idx_type j = 1; j <= m_rep->m_ncols; j++)
         {
           octave_idx_type u = xcidx (j);
           for (; i < u; i++)
@@ -989,26 +1013,27 @@ Sparse<T>::resize (octave_idx_type r, octave_idx_type c)
         }
     }
 
-  rep->nrows = dimensions(0) = r;
+  m_rep->m_nrows = m_dimensions(0) = r;
 
-  if (c != rep->ncols)
+  if (c != m_rep->m_ncols)
     {
       octave_idx_type *new_cidx = new octave_idx_type [c+1];
-      std::copy_n (rep->c, std::min (c, rep->ncols) + 1, new_cidx);
-      delete [] rep->c;
-      rep->c = new_cidx;
+      std::copy_n (m_rep->m_cidx, std::min (c, m_rep->m_ncols) + 1, new_cidx);
+      delete [] m_rep->m_cidx;
+      m_rep->m_cidx = new_cidx;
 
-      if (c > rep->ncols)
-        std::fill_n (rep->c + rep->ncols + 1, c - rep->ncols,
-                     rep->c[rep->ncols]);
+      if (c > m_rep->m_ncols)
+        std::fill_n (m_rep->m_cidx + m_rep->m_ncols + 1, c - m_rep->m_ncols,
+                     m_rep->m_cidx[m_rep->m_ncols]);
     }
 
-  rep->ncols = dimensions(1) = c;
+  m_rep->m_ncols = m_dimensions(1) = c;
 
-  rep->change_length (rep->nnz ());
+  m_rep->change_length (m_rep->nnz ());
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>&
 Sparse<T>::insert (const Sparse<T>& a, octave_idx_type r, octave_idx_type c)
 {
@@ -1032,8 +1057,8 @@ Sparse<T>::insert (const Sparse<T>& a, octave_idx_type r, octave_idx_type c)
         nel++;
 
   Sparse<T> tmp (*this);
-  --rep->count;
-  rep = new typename Sparse<T>::SparseRep (nr, nc, nel);
+  --m_rep->m_count;
+  m_rep = new typename Sparse<T>::SparseRep (nr, nc, nel);
 
   for (octave_idx_type i = 0; i < tmp.cidx (c); i++)
     {
@@ -1090,6 +1115,7 @@ Sparse<T>::insert (const Sparse<T>& a, octave_idx_type r, octave_idx_type c)
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>&
 Sparse<T>::insert (const Sparse<T>& a, const Array<octave_idx_type>& ra_idx)
 {
@@ -1101,6 +1127,7 @@ Sparse<T>::insert (const Sparse<T>& a, const Array<octave_idx_type>& ra_idx)
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>
 Sparse<T>::transpose (void) const
 {
@@ -1140,7 +1167,8 @@ Sparse<T>::transpose (void) const
 // Lower bound lookup.  Could also use octave_sort, but that has upper bound
 // semantics, so requires some manipulation to set right.  Uses a plain loop
 // for small columns.
-static octave_idx_type
+static
+octave_idx_type
 lblookup (const octave_idx_type *ridx, octave_idx_type nr,
           octave_idx_type ri)
 {
@@ -1157,8 +1185,9 @@ lblookup (const octave_idx_type *ridx, octave_idx_type nr,
 }
 
 template <typename T>
+OCTAVE_API
 void
-Sparse<T>::delete_elements (const idx_vector& idx)
+Sparse<T>::delete_elements (const octave::idx_vector& idx)
 {
   Sparse<T> retval;
 
@@ -1201,7 +1230,7 @@ Sparse<T>::delete_elements (const idx_vector& idx)
         {
           OCTAVE_LOCAL_BUFFER (octave_idx_type, ridx_new, nz);
           OCTAVE_LOCAL_BUFFER (T, data_new, nz);
-          idx_vector sidx = idx.sorted (true);
+          octave::idx_vector sidx = idx.sorted (true);
           const octave_idx_type *sj = sidx.raw ();
           octave_idx_type sl = sidx.length (nel);
           octave_idx_type nz_new = 0;
@@ -1250,7 +1279,7 @@ Sparse<T>::delete_elements (const idx_vector& idx)
         *this = Sparse<T> ();
       else
         {
-          *this = index (idx_vector::colon);
+          *this = index (octave::idx_vector::colon);
           delete_elements (idx);
           *this = transpose (); // We want a row vector.
         }
@@ -1258,8 +1287,9 @@ Sparse<T>::delete_elements (const idx_vector& idx)
 }
 
 template <typename T>
+OCTAVE_API
 void
-Sparse<T>::delete_elements (const idx_vector& idx_i, const idx_vector& idx_j)
+Sparse<T>::delete_elements (const octave::idx_vector& idx_i, const octave::idx_vector& idx_j)
 {
   assert (ndims () == 2);
 
@@ -1326,8 +1356,8 @@ Sparse<T>::delete_elements (const idx_vector& idx_i, const idx_vector& idx_j)
           else
             {
               // This is more memory-efficient than the approach below.
-              const Sparse<T> tmpl = index (idx_vector (0, lb), idx_j);
-              const Sparse<T> tmpu = index (idx_vector (ub, nr), idx_j);
+              const Sparse<T> tmpl = index (octave::idx_vector (0, lb), idx_j);
+              const Sparse<T> tmpu = index (octave::idx_vector (ub, nr), idx_j);
               *this = Sparse<T> (nr - (ub - lb), nc,
                                  tmpl.nnz () + tmpu.nnz ());
               for (octave_idx_type j = 0, k = 0; j < nc; j++)
@@ -1377,20 +1407,22 @@ Sparse<T>::delete_elements (const idx_vector& idx_i, const idx_vector& idx_j)
 }
 
 template <typename T>
+OCTAVE_API
 void
-Sparse<T>::delete_elements (int dim, const idx_vector& idx)
+Sparse<T>::delete_elements (int dim, const octave::idx_vector& idx)
 {
   if (dim == 0)
-    delete_elements (idx, idx_vector::colon);
+    delete_elements (idx, octave::idx_vector::colon);
   else if (dim == 1)
-    delete_elements (idx_vector::colon, idx);
+    delete_elements (octave::idx_vector::colon, idx);
   else
     (*current_liboctave_error_handler) ("invalid dimension in delete_elements");
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>
-Sparse<T>::index (const idx_vector& idx, bool resize_ok) const
+Sparse<T>::index (const octave::idx_vector& idx, bool resize_ok) const
 {
   Sparse<T> retval;
 
@@ -1577,7 +1609,7 @@ Sparse<T>::index (const idx_vector& idx, bool resize_ok) const
           // I suppose this is rare (and it may easily overflow), so let's take
           // the easy way, and reshape first to column vector, which is already
           // handled above.
-          retval = index (idx_vector::colon).index (idx);
+          retval = index (octave::idx_vector::colon).index (idx);
           // In this case we're supposed to always inherit the shape, but
           // column(row) doesn't do it, so we'll do it instead.
           if (idx_dims(0) == 1 && idx_dims(1) != 1)
@@ -1589,8 +1621,9 @@ Sparse<T>::index (const idx_vector& idx, bool resize_ok) const
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>
-Sparse<T>::index (const idx_vector& idx_i, const idx_vector& idx_j,
+Sparse<T>::index (const octave::idx_vector& idx_i, const octave::idx_vector& idx_j,
                   bool resize_ok) const
 {
   Sparse<T> retval;
@@ -1783,7 +1816,7 @@ Sparse<T>::index (const idx_vector& idx_i, const idx_vector& idx_j,
       else
         {
           // Get inverse permutation.
-          idx_vector idx_iinv = idx_i.inverse_permutation (nr);
+          octave::idx_vector idx_iinv = idx_i.inverse_permutation (nr);
           const octave_idx_type *iinv = idx_iinv.raw ();
 
           // Scatter buffer.
@@ -1819,22 +1852,23 @@ Sparse<T>::index (const idx_vector& idx_i, const idx_vector& idx_j,
       // so we rely on the efficient transpose to handle this.
       // It may still make sense to optimize some cases here.
       retval = transpose ();
-      retval = retval.index (idx_vector::colon, idx_i);
+      retval = retval.index (octave::idx_vector::colon, idx_i);
       retval = retval.transpose ();
     }
   else
     {
       // A(I, J) is decomposed into A(:, J)(I, :).
-      retval = index (idx_vector::colon, idx_j);
-      retval = retval.index (idx_i, idx_vector::colon);
+      retval = index (octave::idx_vector::colon, idx_j);
+      retval = retval.index (idx_i, octave::idx_vector::colon);
     }
 
   return retval;
 }
 
 template <typename T>
+OCTAVE_API
 void
-Sparse<T>::assign (const idx_vector& idx, const Sparse<T>& rhs)
+Sparse<T>::assign (const octave::idx_vector& idx, const Sparse<T>& rhs)
 {
   Sparse<T> retval;
 
@@ -1866,7 +1900,7 @@ Sparse<T>::assign (const idx_vector& idx, const Sparse<T>& rhs)
 
       if (idx.is_colon ())
         {
-          *this = rhs.reshape (dimensions);
+          *this = rhs.reshape (m_dimensions);
         }
       else if (nc == 1 && rhs.cols () == 1)
         {
@@ -1926,7 +1960,7 @@ Sparse<T>::assign (const idx_vector& idx, const Sparse<T>& rhs)
           else if (idx.is_range () && idx.increment () == -1)
             {
               // It's s(u:-1:l) = r.  Reverse the assignment.
-              assign (idx.sorted (), rhs.index (idx_vector (rhl - 1, 0, -1)));
+              assign (idx.sorted (), rhs.index (octave::idx_vector (rhl - 1, 0, -1)));
             }
           else if (idx.is_permutation (n))
             {
@@ -1957,18 +1991,16 @@ Sparse<T>::assign (const idx_vector& idx, const Sparse<T>& rhs)
               std::copy_n (tmp.data (), nz, new_data.fortran_vec ());
               // ... insert new data (densified) ...
               idx.copy_data (new_ri.fortran_vec () + nz);
-              new_data.assign (idx_vector (nz, new_nz), rhs.array_value ());
+              new_data.assign (octave::idx_vector (nz, new_nz), rhs.array_value ());
               // ... reassembly.
-              *this = Sparse<T> (new_data, new_ri,
-                                 static_cast<octave_idx_type> (0),
-                                 nr, nc, false);
+              *this = Sparse<T> (new_data, new_ri, 0, nr, nc, false);
             }
         }
       else
         {
-          dim_vector save_dims = dimensions;
-          *this = index (idx_vector::colon);
-          assign (idx, rhs.index (idx_vector::colon));
+          dim_vector save_dims = m_dimensions;
+          *this = index (octave::idx_vector::colon);
+          assign (idx, rhs.index (octave::idx_vector::colon));
           *this = reshape (save_dims);
         }
     }
@@ -1981,13 +2013,26 @@ Sparse<T>::assign (const idx_vector& idx, const Sparse<T>& rhs)
         assign (idx, Sparse<T> (rhl, 1));
     }
   else
-    octave::err_nonconformant ("=", dim_vector(idx.length (n),1), rhs.dims());
+    octave::err_nonconformant ("=", dim_vector(idx.length (n), 1), rhs.dims());
 }
 
 template <typename T>
+OCTAVE_API
 void
-Sparse<T>::assign (const idx_vector& idx_i,
-                   const idx_vector& idx_j, const Sparse<T>& rhs)
+Sparse<T>::assign (const octave::idx_vector& idx, const T& rhs)
+{
+  // FIXME: Converting the RHS and forwarding to the sparse matrix
+  // assignment function is simpler, but it might be good to have a
+  // specialization...
+
+  assign (idx, Sparse<T> (1, 1, rhs));
+}
+
+template <typename T>
+OCTAVE_API
+void
+Sparse<T>::assign (const octave::idx_vector& idx_i,
+                   const octave::idx_vector& idx_j, const Sparse<T>& rhs)
 {
   Sparse<T> retval;
 
@@ -2120,7 +2165,7 @@ Sparse<T>::assign (const idx_vector& idx_i,
             {
               // It's s(:,u:-1:l) = r.  Reverse the assignment.
               assign (idx_i, idx_j.sorted (),
-                      rhs.index (idx_i, idx_vector (m - 1, 0, -1)));
+                      rhs.index (idx_i, octave::idx_vector (m - 1, 0, -1)));
             }
           else if (idx_j.is_permutation (nc))
             {
@@ -2191,16 +2236,16 @@ Sparse<T>::assign (const idx_vector& idx_i,
               // especially for many small columns.  OTOH, transpose is an
               // efficient O(nr+nc+nnz) operation.
               *this = transpose ();
-              assign (idx_vector::colon, idx_i, rhs.transpose ());
+              assign (octave::idx_vector::colon, idx_i, rhs.transpose ());
               *this = transpose ();
             }
         }
       else
         {
           // Split it into 2 assignments and one indexing.
-          Sparse<T> tmp = index (idx_vector::colon, idx_j);
-          tmp.assign (idx_i, idx_vector::colon, rhs);
-          assign (idx_vector::colon, idx_j, tmp);
+          Sparse<T> tmp = index (octave::idx_vector::colon, idx_j);
+          tmp.assign (idx_i, octave::idx_vector::colon, rhs);
+          assign (octave::idx_vector::colon, idx_j, tmp);
         }
     }
   else if (m == 1 && n == 1)
@@ -2221,9 +2266,23 @@ Sparse<T>::assign (const idx_vector& idx_i,
     octave::err_nonconformant  ("=", idx_i.length (nr), idx_j.length (nc), n, m);
 }
 
-// Can't use versions of these in Array.cc due to duplication of the
-// instantiations for Array<double and Sparse<double>, etc
 template <typename T>
+OCTAVE_API
+void
+Sparse<T>::assign (const octave::idx_vector& idx_i,
+                   const octave::idx_vector& idx_j, const T& rhs)
+{
+  // FIXME: Converting the RHS and forwarding to the sparse matrix
+  // assignment function is simpler, but it might be good to have a
+  // specialization...
+
+  assign (idx_i, idx_j, Sparse<T> (1, 1, rhs));
+}
+
+// Can't use versions of these in Array.cc due to duplication of the
+// instantiations for Array<double and Sparse<double>, etc.
+template <typename T>
+OCTAVE_API
 bool
 sparse_ascending_compare (typename ref_param<T>::type a,
                           typename ref_param<T>::type b)
@@ -2232,6 +2291,7 @@ sparse_ascending_compare (typename ref_param<T>::type a,
 }
 
 template <typename T>
+OCTAVE_API
 bool
 sparse_descending_compare (typename ref_param<T>::type a,
                            typename ref_param<T>::type b)
@@ -2240,6 +2300,7 @@ sparse_descending_compare (typename ref_param<T>::type a,
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>
 Sparse<T>::sort (octave_idx_type dim, sortmode mode) const
 {
@@ -2251,11 +2312,11 @@ Sparse<T>::sort (octave_idx_type dim, sortmode mode) const
   if (m.numel () < 1 || dim > 1)
     return m;
 
-  if (dim > 0)
+  bool sort_by_column = (dim > 0);
+  if (sort_by_column)
     {
       m = m.transpose ();
-      nr = m.rows ();
-      nc = m.columns ();
+      std::swap (nr, nc);
     }
 
   octave_sort<T> lsort;
@@ -2299,15 +2360,16 @@ Sparse<T>::sort (octave_idx_type dim, sortmode mode) const
       mridx += ns;
     }
 
-  if (dim > 0)
+  if (sort_by_column)
     m = m.transpose ();
 
   return m;
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>
-Sparse<T>::sort (Array<octave_idx_type> &sidx, octave_idx_type dim,
+Sparse<T>::sort (Array<octave_idx_type>& sidx, octave_idx_type dim,
                  sortmode mode) const
 {
   Sparse<T> m = *this;
@@ -2321,11 +2383,11 @@ Sparse<T>::sort (Array<octave_idx_type> &sidx, octave_idx_type dim,
       return m;
     }
 
-  if (dim > 0)
+  bool sort_by_column = (dim > 0);
+  if (sort_by_column)
     {
       m = m.transpose ();
-      nr = m.rows ();
-      nc = m.columns ();
+      std::swap (nr, nc);
     }
 
   octave_sort<T> indexed_sort;
@@ -2353,7 +2415,7 @@ Sparse<T>::sort (Array<octave_idx_type> &sidx, octave_idx_type dim,
       if (ns == 0)
         {
           for (octave_idx_type k = 0; k < nr; k++)
-            sidx (offset + k) = k;
+            sidx(offset + k) = k;
         }
       else
         {
@@ -2383,18 +2445,18 @@ Sparse<T>::sort (Array<octave_idx_type> &sidx, octave_idx_type dim,
               if (ii < ns && mridx[ii] == k)
                 ii++;
               else
-                sidx (offset + jj++) = k;
+                sidx(offset + jj++) = k;
             }
 
           for (octave_idx_type k = 0; k < i; k++)
             {
-              sidx (k + offset) = vi[k];
+              sidx(k + offset) = vi[k];
               mridx[k] = k;
             }
 
           for (octave_idx_type k = i; k < ns; k++)
             {
-              sidx (k - ns + nr + offset) = vi[k];
+              sidx(k - ns + nr + offset) = vi[k];
               mridx[k] = k - ns + nr;
             }
 
@@ -2403,7 +2465,7 @@ Sparse<T>::sort (Array<octave_idx_type> &sidx, octave_idx_type dim,
         }
     }
 
-  if (dim > 0)
+  if (sort_by_column)
     {
       m = m.transpose ();
       sidx = sidx.transpose ();
@@ -2413,6 +2475,7 @@ Sparse<T>::sort (Array<octave_idx_type> &sidx, octave_idx_type dim,
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>
 Sparse<T>::diag (octave_idx_type k) const
 {
@@ -2586,6 +2649,7 @@ Sparse<T>::diag (octave_idx_type k) const
 }
 
 template <typename T>
+OCTAVE_API
 Sparse<T>
 Sparse<T>::cat (int dim, octave_idx_type n, const Sparse<T> *sparse_list)
 {
@@ -2669,7 +2733,7 @@ Sparse<T>::cat (int dim, octave_idx_type n, const Sparse<T> *sparse_list)
               continue;
 
             octave_idx_type u = l + sparse_list[i].columns ();
-            retval.assign (idx_vector::colon, idx_vector (l, u),
+            retval.assign (octave::idx_vector::colon, octave::idx_vector (l, u),
                            sparse_list[i]);
             l = u;
           }
@@ -2684,6 +2748,7 @@ Sparse<T>::cat (int dim, octave_idx_type n, const Sparse<T> *sparse_list)
 }
 
 template <typename T>
+OCTAVE_API
 Array<T>
 Sparse<T>::array_value () const
 {
@@ -2708,6 +2773,7 @@ Sparse<T>::array_value () const
 }
 
 template <typename T>
+OCTAVE_API
 std::istream&
 read_sparse_matrix (std::istream& is, Sparse<T>& a,
                     T (*read_fcn) (std::istream&))
@@ -2969,10 +3035,10 @@ read_sparse_matrix (std::istream& is, Sparse<T>& a,
 %!test test_sparse_slice ([2 2], 22, 3);
 %!test test_sparse_slice ([2 2], 22, 4);
 
-%!assert <35570> (speye (3,1)(3:-1:1), sparse ([0; 0; 1]))
+%!assert <*35570> (speye (3,1)(3:-1:1), sparse ([0; 0; 1]))
 
 ## Test removing columns
-%!test <36656>
+%!test <*36656>
 %! s = sparse (magic (5));
 %! s(:,2:4) = [];
 %! assert (s, sparse (magic (5)(:, [1,5])));
@@ -2983,21 +3049,21 @@ read_sparse_matrix (std::istream& is, Sparse<T>& a,
 %! assert (s, sparse ([], [], [], 0, 1));
 
 ## Test (bug #37321)
-%!test <37321> a=sparse (0,0); assert (all (a) == sparse ([1]));
-%!test <37321> a=sparse (0,1); assert (all (a) == sparse ([1]));
-%!test <37321> a=sparse (1,0); assert (all (a) == sparse ([1]));
-%!test <37321> a=sparse (1,0); assert (all (a,2) == sparse ([1]));
-%!test <37321> a=sparse (1,0); assert (size (all (a,1)), [1 0]);
-%!test <37321> a=sparse (1,1);
+%!test <*37321> a=sparse (0,0); assert (all (a) == sparse ([1]));
+%!test <*37321> a=sparse (0,1); assert (all (a) == sparse ([1]));
+%!test <*37321> a=sparse (1,0); assert (all (a) == sparse ([1]));
+%!test <*37321> a=sparse (1,0); assert (all (a,2) == sparse ([1]));
+%!test <*37321> a=sparse (1,0); assert (size (all (a,1)), [1 0]);
+%!test <*37321> a=sparse (1,1);
 %! assert (all (a) == sparse ([0]));
 %! assert (size (all (a)), [1 1]);
-%!test <37321> a=sparse (2,1);
+%!test <*37321> a=sparse (2,1);
 %! assert (all (a) == sparse ([0]));
 %! assert (size (all (a)), [1 1]);
-%!test <37321> a=sparse (1,2);
+%!test <*37321> a=sparse (1,2);
 %! assert (all (a) == sparse ([0]));
 %! assert (size (all (a)), [1 1]);
-%!test <37321> a=sparse (2,2); assert (isequal (all (a), sparse ([0 0])));
+%!test <*37321> a=sparse (2,2); assert (isequal (all (a), sparse ([0 0])));
 
 ## Test assigning row to a column slice
 %!test <45589>
@@ -3010,21 +3076,30 @@ read_sparse_matrix (std::istream& is, Sparse<T>& a,
 */
 
 template <typename T>
+OCTAVE_API
 void
 Sparse<T>::print_info (std::ostream& os, const std::string& prefix) const
 {
-  os << prefix << "rep address: " << rep << "\n"
-     << prefix << "rep->nzmx:   " << rep->nzmx  << "\n"
-     << prefix << "rep->nrows:  " << rep->nrows << "\n"
-     << prefix << "rep->ncols:  " << rep->ncols << "\n"
-     << prefix << "rep->data:   " << static_cast<void *> (rep->d) << "\n"
-     << prefix << "rep->ridx:   " << static_cast<void *> (rep->r) << "\n"
-     << prefix << "rep->cidx:   " << static_cast<void *> (rep->c) << "\n"
-     << prefix << "rep->count:  " << rep->count << "\n";
+  os << prefix << "m_rep address:  " << m_rep << "\n"
+     << prefix << "m_rep->m_nzmax: " << m_rep->m_nzmax  << "\n"
+     << prefix << "m_rep->m_nrows: " << m_rep->m_nrows << "\n"
+     << prefix << "m_rep->m_ncols: " << m_rep->m_ncols << "\n"
+     << prefix << "m_rep->m_data:  " << m_rep->m_data << "\n"
+     << prefix << "m_rep->m_ridx:  " << m_rep->m_ridx << "\n"
+     << prefix << "m_rep->m_cidx:  " << m_rep->m_cidx << "\n"
+     << prefix << "m_rep->m_count: " << m_rep->m_count << "\n";
 }
 
-#define INSTANTIATE_SPARSE(T, API)                                      \
-  template class API Sparse<T>;                                         \
-  template std::istream&                                                \
-  read_sparse_matrix<T> (std::istream& is, Sparse<T>& a,                \
-                         T (*read_fcn) (std::istream&));
+#if defined (__clang__)
+#  define INSTANTIATE_SPARSE(T)                                         \
+     template class OCTAVE_API Sparse<T>;                               \
+     template OCTAVE_API std::istream&                                  \
+     read_sparse_matrix<T> (std::istream& is, Sparse<T>& a,             \
+                            T (*read_fcn) (std::istream&));
+#else
+#  define INSTANTIATE_SPARSE(T)                                         \
+     template class Sparse<T>;                                          \
+     template OCTAVE_API std::istream&                                  \
+     read_sparse_matrix<T> (std::istream& is, Sparse<T>& a,             \
+                            T (*read_fcn) (std::istream&));
+#endif
