@@ -1,6 +1,6 @@
 ########################################################################
 ##
-## Copyright (C) 2005-2020 The Octave Project Developers
+## Copyright (C) 2005-2022 The Octave Project Developers
 ##
 ## See the file COPYRIGHT.md in the top-level directory of this
 ## distribution or <https://octave.org/copyright/>.
@@ -88,9 +88,9 @@ function configure_make (desc, packdir, verbose)
       cmd = ["cd '" src "'; " scenv " ./configure " flags];
       [status, output] = shell (cmd, verbose);
       if (status != 0)
-        rmdir (desc.dir, "s");
+        sts = rmdir (desc.dir, "s");
         disp (output);
-        error ("pkg: error running the configure script for %s.", desc.name);
+        error ("pkg: error running the configure script for %s", desc.name);
       endif
     endif
 
@@ -105,9 +105,9 @@ function configure_make (desc, packdir, verbose)
       [status, output] = shell (sprintf ("%s make --jobs %i --directory '%s'",
                                          scenv, jobs, src), verbose);
       if (status != 0)
-        rmdir (desc.dir, "s");
+        sts = rmdir (desc.dir, "s");
         disp (output);
-        error ("pkg: error running 'make' for the %s package.", desc.name);
+        error ("pkg: error running 'make' for the %s package", desc.name);
       endif
     endif
 
@@ -169,7 +169,7 @@ function [status, output] = shell (cmd, verbose)
     if (have_sh)
       cmd = ['sh.exe -c "' cmd '"'];
     else
-      error ("pkg: unable to find the command shell.");
+      error ("pkg: unable to find the command shell");
     endif
   endif
   ## if verbose, we want to display the output in real time.  To do this, we
@@ -187,6 +187,7 @@ function [status, output] = shell (cmd, verbose)
 endfunction
 
 function body = __extract_test_code__ (nm)
+
   ## Collect all BIST lines starting %! from the file named nm
   ## and return them as a single \n-delimited string.
   fid = fopen (nm, "rt");
@@ -199,4 +200,5 @@ function body = __extract_test_code__ (nm)
     endwhile
     fclose (fid);
   endif
+
 endfunction
